/***********************************************************************
*                    SEGGER Microcontroller GmbH                       *
*                        The Embedded Experts                          *
************************************************************************
*                                                                      *
*                  (c) SEGGER Microcontroller GmbH                     *
*                        All rights reserved                           *
*                          www.segger.com                              *
*                                                                      *
************************************************************************
*                                                                      *
************************************************************************
*                                                                      *
*                                                                      *
*  Licensing terms                                                     *
*                                                                      *
* Redistribution and use in source and binary forms, with or without   *
* modification, are permitted provided that the following conditions   *
* are met:                                                             *
*                                                                      *
* 1. Redistributions of source code must retain the above copyright    *
* notice, this list of conditions and the following disclaimer.        *
*                                                                      *
* 2. Redistributions in binary form must reproduce the above           *
* copyright notice, this list of conditions and the following          *
* disclaimer in the documentation and/or other materials provided      *
* with the distribution.                                               *
*                                                                      *
*                                                                      *
* THIS SOFTWARE IS PROVIDED BY COPYRIGHT HOLDER "AS IS" AND ANY        *
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE    *
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR   *
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDER BE        *
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,     *
* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,             *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   *
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  *
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         *
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH.    *
* DAMAGE.                                                              *
*                                                                      *
************************************************************************

-------------------------- END-OF-HEADER -----------------------------

File    : RAMCode_SPIFI_Template.c
Purpose : Implementation of RAMCode for <SiliconVendor> <DeviceFamily>

Literature:
  Wiki: XXX
  Reference manual: XXX
  Datasheet: XXX
*/

#include "RAMCode_Intern.h"
#include "SEGGER_SPIFI_Lib.h"

/*********************************************************************
*
*       Defines (configurable)
*
**********************************************************************
*/
#define _SPIFI_LIB_SESSION_FLAGS  (0)
//#define _SPIFI_LIB_SESSION_FLAGS  (SEGGER_SPIFI_LIB_SESSION_FLAG_IS_DUAL_SPIFI)

/*********************************************************************
*
*       Defines (fixed)
*
**********************************************************************
*/
#define _FLASH_BASE_ADDR (0x08000000)

/*********************************************************************
*
*       Types
*
**********************************************************************
*/

//
// Max. 32 bytes in size
//
typedef struct {
  U32 RestoreData0;
  U32 RestoreData1;
  U32 RestoreData2;
  U32 RestoreData3;
  U32 RestoreData4;
  U32 RestoreData5;
  U32 RestoreData6;
  U32 RestoreData7;
} _RESTORE_INFO;

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

//
// Zero-inited data defined in ASM file to have a zero-inited context
// Not really const because the whole code is linked to RAM. But most compilers have no problem with placing RO data pos. independent, while pos. independent RW data is problematic
// Also, this way we have a very elegant zero-initialization for our context...
// Last but not least: This allows us to have cross-function variables that have 0 dependency on the PC side to leave space for them in the RAM, as they are part of the RAMCode
//
extern const SEGGER_SPIFI_LIB_CONTEXT ASM_SPIFILibContext;

/*********************************************************************
*
*       Static code
*
**********************************************************************
*/

/*********************************************************************
*
*       _cb_EnterLeaveSPIMode
*
*  Function description
*    Depending on IsLeave enters SPI mode or leaves SPI mode.
*    Leaving SPI mode usually means entering memory mapped mode.
*
*  Return value
*    >= 0  O.K.
*     < 0  Error
*
*  Notes
*   (1) In case some special settings are necessary to make flash controller work correctly with 4-byte addresses (Flashes > 16 MB)
*       this is the correct routine, as it is always called when switching back to memory mapped mode and so called before we can do some compare etc. stuff.
*/
static void _cb_EnterLeaveSPIMode(U32* pRAMCodeTmpData, int IsLeave, U32 cbFlags, U8 QuadMode) {
  (void)pRAMCodeTmpData;
  (void)cbFlags;
  (void)IsLeave;
  (void)QuadMode;
  //
  // Dummy code, needs to be replaced.
  //
}

/*********************************************************************
*
*       _SendCMD()
*
*  Return value
*    >= 0  O.K.
*     < 0  Error (See SEGGER_SPIFI_LIB_ERR_ defines in SEGGER_SPIFI_Lib.h)
*/
static int _SendCMD(U32 Cmd, U32 Addr, U32 NumBytesAddr, U32 cbFlags) {
  (void)Cmd;
  (void)Addr;
  (void)NumBytesAddr;
  (void)cbFlags;
  //
  // Dummy code, needs to be replaced.
  //
  return 0;
}

/*********************************************************************
*
*       _WriteBytes()
*
*  Return value
*    >= 0  O.K.
*     < 0  Error (See SEGGER_SPIFI_LIB_ERR_ defines in SEGGER_SPIFI_Lib.h)
*/
static int _WriteBytes(U32 Cmd, U32 Addr, U32 NumBytesAddr, void* pData, U32 NumBytesData, U32 cbFlags) {
  (void)Cmd;
  (void)Addr;
  (void)NumBytesAddr;
  (void)pData;
  (void)NumBytesData;
  (void)cbFlags;
//  
// Dummy code below needs to be replacted.
//
//  U8* pData8;
//  U32 v;
//  volatile U8* pTXBuff;
//
//  pData8 = (U8*)pData;
//  //
//  // Right now, we configure the FIFO threshold to 8 bytes so we transmit 8 bytes at once
//  //
//  do {
//    FeedWatchdog();
//    NumBytesFIFOAtOnce = (NumBytesData < _FIFO_NUM_BYTES_AT_ONCE) ? NumBytesData : _FIFO_NUM_BYTES_AT_ONCE;
//    NumBytesData      -= NumBytesFIFOAtOnce;
//    pTXBuff            = (U8*)&_FLEXSPI_TFDR0;
//    do {
//      while(((_FLEXSPI_INTR & (1 << 6)) == 0));    // IP TX FIFO has no less empty space than WaterMark level interrupt.
//      *pTXBuff++ = *pData8++;
//      __asm("DSB");                                // Make sure that all writes made it to the hardware
//      __asm("ISB");                                // Make sure that pipeline is flushed
//    } while (--NumBytesFIFOAtOnce);
//  } while (NumBytesData);
//  //
//  // CS is high at this point
//  //
  return 0;
}

/*********************************************************************
*
*       _ReadBytes()
*
*  Return value
*    >= 0  O.K.
*     < 0  Error (See SEGGER_SPIFI_LIB_ERR_ defines in SEGGER_SPIFI_Lib.h)
*/
static int _ReadBytes(U32 Cmd, void* pData, U32 NumBytesToRead, U32 cbFlags) {
  (void)Cmd;
  (void)pData;
  (void)NumBytesToRead;
  (void)cbFlags;
//  
// Dummy code below needs to be replacted.
//
//  U8* pData8;
//  volatile U8* pRXBuff;
//  U32 NumBytesAtOnce;
//
//  pData8  = (U8*)pData;
//  pRXBuff = (U8*)&_FLEXSPI_RFDR0;
//  do {
//    while(((_FLEXSPI_INTR & (1 << 5)) == 0));    // IP TX FIFO has no less empty space than WaterMark level interrupt.
//    FeedWatchdog();
//    NumBytesAtOnce = _FIFO_NUM_BYTES_AT_ONCE;
//    do {
//      *pData8++ = *pRXBuff++;
//      NumBytesAtOnce--;
//      NumBytesToRead--;
//    } while ((NumBytesAtOnce != 0) && (NumBytesToRead != 0));
//  } while (NumBytesToRead);
//  //
//  // CS is high at this point
//  //
  return 0;
}

/*********************************************************************
*
*       _cbExecCmd()
*
*  Return value
*    >= 0  O.K.
*     < 0  Error (See SEGGER_SPIFI_LIB_ERR_ defines in SEGGER_SPIFI_Lib.h)
*
*  Notes:
*    (1) CS is HIGH at the beginning of the function and needs to he HIGH again at the end.
*/
static int _cbExecCmd(U32 Cmd, U32 Addr, U32 NumBytesAddr, void* pData, U32 NumBytesData, U32 cbFlags, U32* pRAMCodeTmpData) {
  U8  IsWrite;
  U32 NumBytesToRead;
  //
  // The QSPI controller of the NXP iMXRT1170 handles CS automatically so we do not have to set/clear CS explicitly.
  //
  (void)pRAMCodeTmpData;
  //
  // Check if read or write access should be performed
  //
  if (cbFlags & SEGGER_SPIFI_LIB_CB_FLAG_DATA_WRITE) {
    IsWrite = 1;
  } else {
    IsWrite = 0;
  }
  if (NumBytesData == 0) {
    _SendCMD(Cmd, Addr, NumBytesAddr, cbFlags);
  }
  if (IsWrite) {
    _WriteBytes(Cmd, Addr, NumBytesAddr, pData, NumBytesData, cbFlags);
    NumBytesToRead = 0;
  } else {
    NumBytesToRead = NumBytesData;
  }
  if (NumBytesToRead) {
    _ReadBytes(Cmd, pData, NumBytesData, cbFlags);
  }
  return 0;
}

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/
/*********************************************************************
*
*       FeedWatchdog
*
*  Notes
*    (1) This function CANNOT rely on that Prepare() has been called previously
*/
void FeedWatchdog(void) {
  // TBD
}

/*********************************************************************
*
*       GetDeviceDesc
*
*  Function description
*    Get flash parameters such as min. alignment of the algorithm, flash size, block size, ...
*
*  Notes
*    (1) This function can rely on that Prepare() has been called previously
*/
int GetDeviceDesc(void* pBuffer, U32 BufferSize) {
  RAMCODE_SPIFI_DEVICE_DESC* pDesc;
  RAMCODE_FLASH_BLOCK* paBlockDesc;
  SEGGER_SPIFI_LIB_CONTEXT* pContext;
  U32 NumBlocks;

  (void)BufferSize;
  pContext = (SEGGER_SPIFI_LIB_CONTEXT*)&ASM_SPIFILibContext;
  if (pContext->Int.NumSectors) {
    pDesc                     = (RAMCODE_SPIFI_DEVICE_DESC*)pBuffer;
    paBlockDesc               = &pDesc->BlockInfo;                                 // Really describes the first element of an array of blocks
    pDesc->Flags              = 0 | RAMCODE_SPIFI_SUPPORT_FLAG_MULTI_SECTOR_ERASE | RAMCODE_SPIFI_SUPPORT_FLAG_MULTI_SECTOR_PROG;
    pDesc->Flags             |= (pContext->Int.MinAlignProg << 0);                 // MinAlignShift
    NumBlocks                 = 1;
    pDesc->NumBlocks          = NumBlocks;
    paBlockDesc->Offset       = 0;
    paBlockDesc->NumSectors   = pContext->Int.NumSectors;
    paBlockDesc->SectorSize   = (1uL << pContext->Int.SectorSizeShift);
  } else {
    NumBlocks = 0;
  }
  return NumBlocks;
}

/*********************************************************************
*
*       Program
*
* Purpose:
*   This function programs the flash memory in units of 16-bits.
*
* Parameters:
*   p->In.pSrc              Source address (Data to be programmed)
*   p->In.pDest             Destination address
*   p->In.NumItems          Specifies the number of 16-bit items. Value is always a multiple of 2.
*   p->In.WritePara0        (optional, not used by all flashes)
*   p->In.WritePara1        (optional, not used by all flashes)
*   p->In.WritePara2        (optional, not used by all flashes)
*
*   p->Out.FailAddr         Contains address of item that caused an error (only used in case of error)
*
* Return values:
*    0 = Operation completed successfully
*    1 = Block verification error (Detected by generic RAMCode, do not use!)
*    2 = Item verification error
*    3 = Timeout occurred
*    4 = Program error
*    5 = Program 1 over 0
*    6 = Sector is locked
*    7 = Erase error
*    8 = No flash memory
*   >8 = Unspecified error (reserved for further error codes)
*   <0 = Algo depending error code (usually a small integer such as -1, -2, -3)
*
*  Notes
*    (1) This function can rely on that Prepare() has been called previously
*/
int Program(IO_PARA* p) {
  SEGGER_SPIFI_LIB_CONTEXT* pContext;
  int r;
  U32 NumBytes;
  U32 Addr;

  pContext = (SEGGER_SPIFI_LIB_CONTEXT*)&ASM_SPIFILibContext;
  Addr     = (U32)p->In.pDest - _FLASH_BASE_ADDR;
  NumBytes = p->In.NumHWords << 1;
  r = SEGGER_SPIFI_LIB_Program(pContext, Addr, NumBytes, p->In.pSrc);  // SPIFI does not know anything about base address of SPIFI area. First sector of flash has addr 0x0
  if (r != 0) {
    return r;
  }
  //
  // Skip verify after program in order to save some time.
  // The DLL / J-Flash performs a verify later anyhow.
  //
// Not supported yet for Open Flashloader. Will be supported from J-Link software >= V6.96.
//  return RAMCODE_RESULT_ERROR_PROGRAM_NO_AUTO_BLOCK_VERIFY_ALLOWED;
  return 0;
}

/*********************************************************************
*
*       Erase
*
* Purpose:
*   This function erases one or more flash sectors or performs a chip erase.
*
* Parameters:
*   NumSectors              NumSectors (0 means chip erase)
*   ErasePara0              (optional, not used by all flashes)
*   ErasePara1              (optional, not used by all flashes)
*   ErasePara2              (optional, not used by all flashes)
*
* Return values:
*    0 = Operation completed successfully
*    1 = Block verification error (Detected by generic RAMCode, do not use!)
*    2 = Item verification error
*    3 = Timeout occurred
*    4 = Program error
*    5 = Program 1 over 0
*    6 = Sector is locked
*    7 = Erase error
*    8 = No flash memory
*   >8 = Unspecified error (reserved for further error codes)
*   <0 = Algo depending error code (usually a small integer such as -1, -2, -3)
*
*  Notes
*    (1) This function can rely on that Prepare() has been called previously
*/
int Erase(U32 NumSectors, U32 SectorAddr, U32 ErasePara1,  U32 ErasePara2) {
  SEGGER_SPIFI_LIB_CONTEXT* pContext;
  int r;

  (void)ErasePara1;
  (void)ErasePara2;
  SectorAddr -= _FLASH_BASE_ADDR;
  pContext = (SEGGER_SPIFI_LIB_CONTEXT*)&ASM_SPIFILibContext;
  r = SEGGER_SPIFI_LIB_Erase(pContext, SectorAddr, NumSectors);  // SPIFI does not know anything about base address of SPIFI area. First sector of flash has addr 0x0
  return r;
}

/*********************************************************************
*
*       Prepare
*
*  Notes
*    (1) Restore info area is 32 bytes in size.
*
* Return values:
*   >0 = OK, clockspeed [Hz] after prepare operation
*    0 = OK, clockspeed not changed by prepare operation
*   <0 = Error
*/
int Prepare(void* pRestoreInfo) {
  _RESTORE_INFO* p;
  SEGGER_SPIFI_LIB_CONTEXT* pContext;
  int r;
 
  p = (_RESTORE_INFO*)pRestoreInfo;
  FeedWatchdog();
  //
  // Dummy code, needs to be replaced with init code (typically clock init, pin init and QSPI controller init)
  //
  //
  // Init SPIFI lib if necessary
  //
  pContext = (SEGGER_SPIFI_LIB_CONTEXT*)&ASM_SPIFILibContext;
  if (pContext->Pub.RAMCodeTmpData == 0) {
    pContext->Pub.pfExecCmd              = _cbExecCmd;
    pContext->Pub.pfOptEnterLeaveSPIMode = _cb_EnterLeaveSPIMode;
    r                                    = SEGGER_SPIFI_LIB_Init(pContext, _SPIFI_LIB_SESSION_FLAGS);
    pContext->Pub.RAMCodeTmpData         = 1;
  } else {
    r = 0;
  }
  return r;
}

/*********************************************************************
*
*       Restore
*
*  Notes
*    (1) Restore info area is 32 bytes in size.
*
* Return values:
*  >=0 = OK
*   <0 = Error
*/
int Restore(void* pRestoreInfo) {
  _RESTORE_INFO* p;
  SEGGER_SPIFI_LIB_CONTEXT* pContext;
  int r;
  //
  // Make sure we de-init the SPIFI lib, in case some things like "quad bit enable" must be restored because the lib has cleared it etc.
  //
  p = (_RESTORE_INFO*)pRestoreInfo;
  pContext = (SEGGER_SPIFI_LIB_CONTEXT*)&ASM_SPIFILibContext;
  if (pContext->Pub.RAMCodeTmpData) {
    r = SEGGER_SPIFI_LIB_UnInit(pContext, _SPIFI_LIB_SESSION_FLAGS);
  } else {
    r = 0;
  }
  p = (_RESTORE_INFO*)pRestoreInfo;
  //
  // Dummy code, needs to be replaced with restore code (typically clock restore, pin restore and QSPI controller restore)
  //
  return r;
}

/**************************** End of file ***************************/
